package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.core.oe.task.service.ExamBossService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamBoss;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description 清理考试记录缓存
 * @Author lideyin
 * @Date 2019/12/24 18:07
 * @Version 1.0
 */
@Component
public class ClearExamDataCacheExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private ExamBossService examBossService;
    private static final ExamCloudLog LOG = ExamCloudLogFactory.getLog(ClearExamDataCacheExecutor.class);

    /**
     * 执行
     *
     * @param key
     * @param examRecordData
     * @param outList
     * @param removable
     * @param context
     * @throws Exception
     * @author WANGWEI
     */
    @Override
    public void execute(Long key, ExamRecordData examRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {

        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + examRecordData.getStudentId();

        try {
            debugLog("enter executor...", examRecordData.getId());

            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            debugLog("get locker success...", examRecordData.getId());

            //只有已同步成功的数据，才执行清理操作
            if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {

                clearExamCache(examRecordData.getExamStudentId());

                this.debugLog("all is over.", examRecordData.getId());

                return;
            }

            this.debugLog("current status is '" + examRecordData.getExamRecordStatus().name() +
                    "'.may be redundant data.", examRecordData.getId());

        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
        }
    }

    /**
     * 考试id
     *
     * @param examId
     */
    private void clearExamCache(Long examId) {
        ExamBoss examBoss = examBossService.getExamBoss(examId);

        if (null == examBoss) {
            return;
        }

        //如果开考次数==考试完结次数，则删除考试基础信息缓存
        if (examBoss.getStartCount() == examBoss.getEndCount()) {
            examBossService.deleteExamBoss(examId);
        }
    }

    private void debugLog(String msg, Long examRecordDataId) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[CLEAR-EXAM-DATA-CACHE-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }
}
